"""CLI tests for jav."""

# from jav.utils import test
#
# class CliTestCase(test.javTestCase):
#     def test_jav_cli(self):
#         argv = ['--foo=bar']
#         with self.make_app(argv=argv) as app:
#             app.run()
#             self.eq(app.pargs.foo, 'bar')
